@@ -24,6 +24,7 @@
 #include "panel-util.h"
 #include "drawer-widget.h"
 #include "gnome-run.h"
+#include "scroll-menu.h"
 
 #define SMALL_ICON_SIZE 20
 
@@ -32,9 +33,12 @@ extern GList *panel_list;
 
 extern GtkTooltips *panel_tooltips;
 
-static void foobar_widget_class_init (FoobarWidgetClass *klass);
-static void foobar_widget_init (FoobarWidget *foo);
-static void foobar_widget_realize (GtkWidget *w);
+static void foobar_widget_class_init	(FoobarWidgetClass	*klass);
+static void foobar_widget_init		(FoobarWidget		*foo);
+static void foobar_widget_realize	(GtkWidget		*w);
+static void foobar_widget_destroy	(GtkObject		*o);
+static void foobar_widget_size_allocate	(GtkWidget		*w,
+					 GtkAllocation		*alloc);
 
 static GtkWidget *das_global_foobar = NULL;
 static GtkWidget *clock_ebox = NULL;
@@ -69,10 +73,14 @@ static void
 foobar_widget_class_init (FoobarWidgetClass *klass)
 {
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+	GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
 
         parent_class = gtk_type_class (gtk_window_get_type ());
 
+	object_class->destroy = foobar_widget_destroy;
+
 	widget_class->realize = foobar_widget_realize;
+	widget_class->size_allocate = foobar_widget_size_allocate;
 }
 
 static GtkWidget *
@@ -164,7 +172,7 @@ gnomecal_client (GtkWidget *w, gpointer data)
 }
 
 static GtkWidget *
-append_gnome_menu (GtkWidget *menu_bar)
+append_gnome_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 {
 	GtkWidget *item;
 	GtkWidget *menu;
@@ -181,7 +189,7 @@ append_gnome_menu (GtkWidget *menu_bar)
 	};
 	
 	
-	menu = gtk_menu_new ();
+	menu = scroll_menu_new ();
 	
 	for (i=0; url[i][1]; i++)
 		gtk_menu_append (GTK_MENU (menu),
@@ -229,7 +237,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 		NULL
 	};
 
-	menu = gtk_menu_new ();
+	menu = scroll_menu_new ();
 
 	for (i=0; arrange[i]; i+=2)
 		append_gmc_item (menu, _(arrange[i]), arrange[i+1]);
@@ -239,7 +247,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 
 	add_tearoff (GTK_MENU (menu));
 
-	menu = gtk_menu_new ();
+	menu = scroll_menu_new ();
 
 	gtk_menu_append (GTK_MENU (menu), item);
 	add_menu_separator (menu);
@@ -274,7 +282,7 @@ append_desktop_menu (GtkWidget *menu_bar)
 	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
 }
 
-static void
+static GtkWidget *
 append_folder_menu (GtkWidget *menu_bar, const char *label,
 		    const char *pixmap, gboolean system, const char *path)
 {
@@ -285,17 +293,17 @@ append_folder_menu (GtkWidget *menu_bar, const char *label,
 		? gnome_unconditional_datadir_file (path)
 		: gnome_util_home_file (path);
 
-	if (!real_path) {
+	if (real_path == NULL) {
 		g_warning (_("can't fine real path"));
-		return;
+		return NULL;
 	}
 
 	menu = create_fake_menu_at (real_path, FALSE, label, NULL, FALSE);
 	g_free (real_path);
 
-	if (!menu) {
+	if (menu == NULL) {
 		g_warning (_("menu wasn't created"));
-		return;
+		return NULL;
 	}
 
 	if (pixmap)
@@ -308,6 +316,8 @@ append_folder_menu (GtkWidget *menu_bar, const char *label,
 	gtk_signal_connect (GTK_OBJECT (menu), "show",
 			    GTK_SIGNAL_FUNC (submenu_to_display),
 			    NULL);
+
+	return menu;
 }
 
 static void
@@ -319,41 +329,58 @@ append_gnomecal_item (GtkWidget *menu, const char *label, const char *flag)
 			    GTK_SIGNAL_FUNC (gnomecal_client), (gpointer)flag);
 }
 
-static int
-timeout_cb (gpointer data)
+static void
+update_clock (FoobarWidget *foo)
 {
 	static int day = 0;
-	GtkWidget *label = GTK_WIDGET (data);
 	struct tm *das_tm;
 	time_t das_time;
-	char hour[20];
+	char hour[256];
 
-	if (!IS_FOOBAR_WIDGET (das_global_foobar))
-		return FALSE;
+	if (foo->clock_label == NULL)
+		return;
 
 	time (&das_time);
 	das_tm = localtime (&das_time);
 
 	if (das_tm->tm_mday != day) {
-		if (strftime (hour, 20, _("%A %B %d"), das_tm) == 20)
-			hour[19] = '\0';
+		if(strftime(hour, sizeof(hour), _("%A %B %d"), das_tm) == 0) {
+			/* according to docs, if the string does not fit, the
+			 * contents of tmp2 are undefined, thus just use
+			 * ??? */
+			strcpy(hour, "???");
+		}
+		hour[sizeof(hour)-1] = '\0'; /* just for sanity */
+
 		gtk_tooltips_set_tip (panel_tooltips, clock_ebox, hour, NULL);
 
 		day = das_tm->tm_mday;
 	}
 
-	if (strftime (hour, 20, FOOBAR_WIDGET (das_global_foobar)->clock_format, das_tm) == 20)
-		hour[19] = '\0';
+	if(strftime(hour, sizeof(hour), foo->clock_format, das_tm) == 0) {
+		/* according to docs, if the string does not fit, the
+		 * contents of tmp2 are undefined, thus just use
+		 * ??? */
+		strcpy(hour, "???");
+	}
+	hour[sizeof(hour)-1] = '\0'; /* just for sanity */
 
-	gtk_label_set_text (GTK_LABEL (label), hour);
-
-	return TRUE;
+	gtk_label_set_text (GTK_LABEL (foo->clock_label), hour);
 }
 
-static void
-timeout_remove (GtkWidget *w, gpointer data)
+static int
+timeout_cb (gpointer data)
 {
-	gtk_timeout_remove (GPOINTER_TO_INT (data));
+	FoobarWidget *foo = FOOBAR_WIDGET (data);
+
+	if (foo->clock_label == NULL) {
+		foo->clock_timeout = 0;
+		return FALSE;
+	}
+
+	update_clock (foo);
+
+	return TRUE;
 }
 
 static void
@@ -362,21 +389,25 @@ set_fooclock_format (GtkWidget *w, char *format)
 	if (!IS_FOOBAR_WIDGET (das_global_foobar))
 		return;
 
-	g_free (FOOBAR_WIDGET (das_global_foobar)->clock_format);
-	FOOBAR_WIDGET (das_global_foobar)->clock_format = g_strdup (_(format));
+	foobar_widget_set_clock_format (FOOBAR_WIDGET (das_global_foobar), _(format));
 }
 
 static void
 append_format_item (GtkWidget *menu, const char *format)
 {
-	char hour[20];
+	char hour[256];
 	GtkWidget *item;
 	struct tm *das_tm;
 	time_t das_time = 0;
 
 	das_tm = localtime (&das_time);
-	if (strftime (hour, 20, _(format), das_tm) == 20)
-		hour[19] = '\0';
+	if (strftime (hour, sizeof(hour), _(format), das_tm) == 0) {
+		/* according to docs, if the string does not fit, the
+		 * contents of tmp2 are undefined, thus just use
+		 * ??? */
+		strcpy(hour, "???");
+	}
+	hour[sizeof(hour)-1] = '\0'; /* just for sanity */
 
 	item = gtk_menu_item_new_with_label (hour);
 	gtk_menu_append (GTK_MENU (menu), item);
@@ -386,10 +417,9 @@ append_format_item (GtkWidget *menu, const char *format)
 }
 
 static GtkWidget *
-append_clock_menu (GtkWidget *menu_bar)
+append_clock_menu (FoobarWidget *foo, GtkWidget *menu_bar)
 {
-	GtkWidget *item, *menu, *label, *menu2;
-	gint timeout;
+	GtkWidget *item, *menu, *menu2;
 	int i;
 	const char *cals[] = { 
 		N_("Today"),      "dayview",
@@ -406,7 +436,7 @@ append_clock_menu (GtkWidget *menu_bar)
 		NULL
 	};
 
-	menu = gtk_menu_new ();
+	menu = scroll_menu_new ();
 	
 #if 0 /* put back when evolution can do this */
 	item = gtk_menu_item_new_with_label (_("Add appointement..."));
@@ -420,7 +450,7 @@ append_clock_menu (GtkWidget *menu_bar)
 
 	add_menu_separator (menu);
 
-	menu2 = gtk_menu_new ();
+	menu2 = scroll_menu_new ();
 	for (i=0; formats[i]; i++)
 		append_format_item (menu2, formats[i]);
 
@@ -433,13 +463,11 @@ append_clock_menu (GtkWidget *menu_bar)
 	add_tearoff (GTK_MENU (menu));
 
 	item = gtk_menu_item_new ();
-	label = gtk_label_new ("");
-	timeout = gtk_timeout_add (1000, timeout_cb, label);
-	gtk_signal_connect (GTK_OBJECT (label), "destroy",
-			    GTK_SIGNAL_FUNC (timeout_remove),
-			    GINT_TO_POINTER (timeout));
+	foo->clock_label = gtk_label_new ("");
+	foo->clock_timeout = gtk_timeout_add (1000, timeout_cb, foo);
+
 	clock_ebox = item;
-	gtk_container_add (GTK_CONTAINER (item), label);
+	gtk_container_add (GTK_CONTAINER (item), foo->clock_label);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), menu);
 
 	gtk_menu_bar_append (GTK_MENU_BAR (menu_bar), item);
@@ -447,6 +475,15 @@ append_clock_menu (GtkWidget *menu_bar)
 	return item;
 }
 
+void
+foobar_widget_set_clock_format (FoobarWidget *foo, const char *clock_format)
+{
+	g_free (foo->clock_format);
+	foo->clock_format = g_strdup (clock_format);
+
+	update_clock (foo);
+}
+
 void
 foobar_widget_update_winhints (FoobarWidget *foo)
 {
@@ -473,11 +510,11 @@ foobar_widget_update_winhints (FoobarWidget *foo)
 static void
 foobar_widget_realize (GtkWidget *w)
 {
-	if(GTK_WIDGET_CLASS(parent_class)->realize)
-		GTK_WIDGET_CLASS(parent_class)->realize(w);
+	if (GTK_WIDGET_CLASS (parent_class)->realize)
+		GTK_WIDGET_CLASS (parent_class)->realize (w);
 
-	foobar_widget_update_winhints(FOOBAR_WIDGET(w));
-	xstuff_set_no_group(w->window);
+	foobar_widget_update_winhints (FOOBAR_WIDGET (w));
+	xstuff_set_no_group (w->window);
 }
 
 static void
@@ -509,6 +546,10 @@ foobar_widget_init (FoobarWidget *foo)
 	/*GtkWidget *align;*/
 	gint flags;
 
+	foo->clock_format = g_strdup (_("%H:%M"));
+	foo->clock_timeout = 0;
+	foo->clock_label = NULL;
+
 	foo->compliant_wm = xstuff_is_compliant_wm ();
 	if(foo->compliant_wm)
 		GTK_WINDOW(foo)->type = GTK_WINDOW_TOPLEVEL;
@@ -554,10 +595,14 @@ foobar_widget_init (FoobarWidget *foo)
 	gtk_signal_connect (GTK_OBJECT (menu), "show",
 			    GTK_SIGNAL_FUNC (programs_menu_to_display),
 			    NULL);
+	foo->programs = menu;
 
-	append_folder_menu  (menu_bar, _("Favorites"), NULL, FALSE, "apps/.");
-	append_folder_menu  (menu_bar, _("Settings"),  NULL, TRUE,
-			     "gnome/apps/Settings/.");
+	foo->favorites =
+		append_folder_menu(menu_bar, _("Favorites"), NULL,
+				   FALSE, "apps");
+	foo->settings =
+		append_folder_menu(menu_bar, _("Settings"),  NULL, TRUE,
+			           "gnome/apps/Settings");
 	append_desktop_menu (menu_bar);
 
 	gtk_box_pack_start (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
@@ -586,8 +631,8 @@ foobar_widget_init (FoobarWidget *foo)
 	menu_bar = gtk_menu_bar_new ();
 	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (menu_bar),
 				      GTK_SHADOW_NONE);
-	append_clock_menu (menu_bar);
-	append_gnome_menu (menu_bar);
+	append_clock_menu (foo, menu_bar);
+	append_gnome_menu (foo, menu_bar);
 
 
 	gtk_box_pack_end (GTK_BOX (foo->hbox), menu_bar, FALSE, FALSE, 0);
@@ -612,31 +657,46 @@ queue_panel_resize (gpointer data, gpointer user_data)
 }
 
 static void
-queue_panel_resizes (GtkWidget *w, gpointer data)
+foobar_widget_destroy (GtkObject *o)
 {
-	if (!GTK_WIDGET_REALIZED (w))
-		return;
-	g_list_foreach (panel_list, queue_panel_resize, data);
+	FoobarWidget *foo = FOOBAR_WIDGET (o);
+
+	/* Just sanity */
+	if ((gpointer)das_global_foobar == (gpointer)foo)
+		das_global_foobar = NULL;
+
+	if (foo->clock_timeout != 0)
+		gtk_timeout_remove (foo->clock_timeout);
+	foo->clock_timeout = 0;
+	
+	foo->clock_label = NULL;
+
+	g_free (foo->clock_format);
+	foo->clock_format = NULL;
+
+	g_list_foreach (panel_list, queue_panel_resize, NULL);
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		GTK_OBJECT_CLASS (parent_class)->destroy (o);
 }
 
 static void
-foobar_destroyed(GtkWidget *w, gpointer data)
+foobar_widget_size_allocate (GtkWidget *w, GtkAllocation *alloc)
 {
-	das_global_foobar = NULL;
-	g_list_foreach (panel_list, queue_panel_resize, data);
+	if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
+		GTK_WIDGET_CLASS (parent_class)->size_allocate (w, alloc);
+
+	if (GTK_WIDGET_REALIZED (w))
+		g_list_foreach (panel_list, queue_panel_resize, NULL);
 }
 
 GtkWidget *
 foobar_widget_new (void)
 {
 	g_return_val_if_fail (das_global_foobar == NULL, NULL);
-	das_global_foobar =  gtk_type_new (FOOBAR_WIDGET_TYPE);
-	gtk_signal_connect_after (GTK_OBJECT (das_global_foobar),
-				  "size-allocate",
-				  GTK_SIGNAL_FUNC (queue_panel_resizes), NULL);
-	gtk_signal_connect (GTK_OBJECT (das_global_foobar), "destroy",
-			    GTK_SIGNAL_FUNC (foobar_destroyed),
-			    NULL);
+
+	das_global_foobar = gtk_type_new (TYPE_FOOBAR_WIDGET);
+
 	return das_global_foobar;
 }
 
@@ -646,6 +706,26 @@ foobar_widget_exists (void)
 	return (das_global_foobar != NULL);
 }
 
+void
+foobar_widget_force_menu_remake (void)
+{
+	FoobarWidget *foo;
+	if (das_global_foobar == NULL)
+		return;
+
+	foo = FOOBAR_WIDGET(das_global_foobar);
+
+	if (foo->programs != NULL)
+		gtk_object_set_data (GTK_OBJECT(foo->programs),
+				     "need_reread", GINT_TO_POINTER(1));
+	if (foo->settings != NULL)
+		gtk_object_set_data (GTK_OBJECT(foo->settings),
+				     "need_reread", GINT_TO_POINTER(1));
+	if (foo->favorites != NULL)
+		gtk_object_set_data (GTK_OBJECT(foo->favorites),
+				     "need_reread", GINT_TO_POINTER(1));
+}
+
 gint
 foobar_widget_get_height (void)
 {
@@ -657,12 +737,12 @@ static void
 reparent_button_widgets(GtkWidget *w, gpointer data)
 {
 	GdkWindow *newwin = data;
-	if(IS_BUTTON_WIDGET(w)) {
+	if (IS_BUTTON_WIDGET (w)) {
 		ButtonWidget *button = BUTTON_WIDGET(w);
 		/* we can just reparent them all to 0,0 as the next thing
 		 * that will happen is a queue_resize and on size allocate
 		 * they will be put into their proper place */
-		gdk_window_reparent(button->event_window, newwin, 0, 0);
+		gdk_window_reparent (button->event_window, newwin, 0, 0);
 	}
 }
 
@@ -678,14 +758,14 @@ foobar_widget_redo_window(FoobarWidget *foo)
 	gboolean comp;
 
 	comp = xstuff_is_compliant_wm();
-	if(comp == foo->compliant_wm)
+	if (comp == foo->compliant_wm)
 		return;
 
 	window = GTK_WINDOW(foo);
 	widget = GTK_WIDGET(foo);
 
 	foo->compliant_wm = comp;
-	if(foo->compliant_wm) {
+	if (foo->compliant_wm) {
 		window->type = GTK_WINDOW_TOPLEVEL;
 		attributes.window_type = GDK_WINDOW_TOPLEVEL;
 	} else {
@@ -693,7 +773,7 @@ foobar_widget_redo_window(FoobarWidget *foo)
 		attributes.window_type = GDK_WINDOW_TEMP;
 	}
 
-	if(!widget->window)
+	if (widget->window == NULL)
 		return;
 
 	/* this is mostly copied from gtkwindow.c realize method */
